import aiohttp
from typing import Optional, Dict, Any, Tuple

async def http_request(
    method: str,
    url: str,
    headers: Optional[Dict[str, str]] = None,
    data: Optional[Any] = None,
    json_data: Optional[Any] = None,
    allow_redirects: bool = True,
    **kwargs
) -> Tuple[str, Dict[str, str], aiohttp.ClientResponse]:
    """
    Make an HTTP request using aiohttp and return (response_text, cookies, response_obj).
    """
    async with aiohttp.ClientSession() as session:
        req_func = getattr(session, method.lower())
        try:
            async with req_func(
                url,
                headers=headers,
                data=data,
                json=json_data,
                allow_redirects=allow_redirects,
                **kwargs
            ) as response:
                text = await response.text()
                cookies = {k: v.value for k, v in response.cookies.items()}
                return text, cookies, response
        except Exception as e:
            # Optionally log or handle exceptions here
            raise


"""API for Lux Thermostat."""

import json
from .auth import login
from .helpers import http_request

class LuxAPI:
    """API for Lux Thermostat."""

    def __init__(self, username, password, tokens) -> None:
        """Initialize the API."""
        self.access_token = tokens["access_token"]
        self.refresh_token = tokens["refresh_token"]
        self.username = username
        self.password = password

    async def get_user(self) -> dict:
        """Get the user."""
        url = "https://www.myluxstat.io/api/location/user"
        headers = {"Authorization": f"Bearer {self.access_token}"}
        res, _, _ = await self._make_request("get", url, headers=headers)
        return json.loads(res)

    async def get_device_state(self, device_id) -> dict:
        """Get the device state."""
        url = "https://www.myluxstat.io/api/device"
        headers = {
            "Authorization": f"Bearer {self.access_token}",
            "Deviceid": device_id,
        }
        res, _, _ = await self._make_request("get", url, headers=headers)
        print(res)
        return json.loads(res)

    async def set_device_state(self, device_id, state) -> dict:
        """Set the device state."""
        url = "https://www.myluxstat.io/api/device"
        headers = {
            "Authorization": f"Bearer {self.access_token}",
            "Deviceid": device_id,
        }
        res, _, _ = await self._make_request("put", url, headers=headers, json_data=state)
        return json.loads(res)

    async def _make_request(self, method, url, headers, data=None, json_data=None):
        res, _, _ = await http_request(method, url, headers=headers, data=data, json_data=json_data)
        j = json.loads(res)
        if "statusCode" in j and j["statusCode"] == 401:
            tokens = await login(self.username, self.password)
            print("here", tokens)
            self.access_token = tokens["access_token"]
            self.refresh_token = tokens["refresh_token"]
            headers["Authorization"] = f"Bearer {self.access_token}"
            return await http_request(method, url, headers, json_data=json_data)
        return res, None, None # return res here instead of j because we already decode in the caller



import asyncio
import json
import os

import dotenv

from .api import LuxAPI
from .auth import login

dotenv.load_dotenv()

async def main():
    # tokens = await login(os.getenv("USERNAME"), os.getenv("PASSWORD"))
    tokens = json.loads(os.getenv("TOKENS"))

    api = LuxAPI(os.getenv("USERNAME"), os.getenv("PASSWORD"), tokens)
    u = await api.get_user()

    device_id = u["location"][0]["devices"][0]["id"]
    print(device_id)
    d = await api.get_device_state(device_id)
    # d["systemmode"] = 0 # 1 for on
    # await api.set_device_state(device_id, d)


if __name__ == "__main__":
    asyncio.run(main())


"""Authentication for Lux Thermostat."""

import json

import aiohttp
import pkce


async def login(username, password) -> dict:
    """Login to Lux Thermostat."""
    cv, cc = pkce.generate_pkce_pair()

    # FIXME factor these out?
    client_id = "b335ca43-3bde-4406-b281-8816afb7cc91"
    redirect_uri = "connecteddevicesjci.luxmobile://connecteddevicesjci/path"

    scope = "https://connecteddevicesjci.onmicrosoft.com/mobile/user_impersonation https://connecteddevicesjci.onmicrosoft.com/mobile/read_write offline_access openid"

    # FIXME nonce should be random
    url = (
        "https://connecteddevicesjci.b2clogin.com/te/connecteddevicesjci.onmicrosoft.com/B2C_1A_SignIn/oauth2/v2.0/authorize?"
        "nonce=FZ7xNNW3u6Gy-LMCrtntZ0yveaUvYRUtVRPPk9gneKM"
        "&audience=https://connecteddevicesjci.onmicrosoft.com"
        f"&scope={scope}"
        f"&response_type=code&client_id={client_id}"
        f"&code_challenge={cc}"
        f"&code_challenge_method=S256&redirect_uri={redirect_uri}"
        "&state=4rwh1Bs9s6OtdGfH0OWww_Y7qDfkdfxiOxRO8hpflFI"
    )  # FIXME state should be random

    res = ""
    cookies = []
    async with aiohttp.ClientSession() as session, session.get(url) as response:
        res = await response.text()
        cookies = response.cookies
        csrf = response.headers.get("X-Csrf-Token")

    csrf = (
        str(cookies.get("x-ms-cpim-csrf"))
        .rsplit("Set-Cookie: x-ms-cpim-csrf=", maxsplit=1)[-1]
        .split(";")[0]
    )

    cookie_dict = {}
    for k in cookies:
        cookie_dict[k] = str(cookies.get(k)).split("=", 1)[-1].split(";")[0].strip('"')

    try:
        split_result = res.split('"transId":"StateProperties=')
        if len(split_result) < 2:
            # print("Error: Could not find StateProperties in response: ", split_result)
            return None
        StateProperties = split_result[1].split('"')[0]
    except IndexError:
        # print("Error: Malformed response when trying to extract StateProperties")
        return None

    cookies = []
    async with aiohttp.ClientSession() as session:
        url2 = f"https://connecteddevicesjci.b2clogin.com/connecteddevicesjci.onmicrosoft.com/B2C_1A_SignIn/SelfAsserted?tx=StateProperties={StateProperties}&p=B2C_1A_SignIn"
        data = {
            "request_type": "RESPONSE",
            "logonIdentifier": username,
            "password": password,
        }
        headers = {"Referer": url, "X-Csrf-Token": csrf}
        # The cookie values are being wrapped in quotes by aiohttp's cookie handling
        # We can pass the raw cookie string in the headers instead
        cookie_str = "; ".join(f"{k}={v}" for k, v in cookie_dict.items())
        headers["Cookie"] = cookie_str
        async with session.post(url2, data=data, headers=headers) as response:
            res = await response.text()
            cookies = response.cookies

    for k in cookies:
        cookie_dict[k] = str(cookies.get(k)).split("=", 1)[-1].split(";")[0].strip('"')

    redirect_url = ""
    async with aiohttp.ClientSession() as session:
        url3 = f"https://connecteddevicesjci.b2clogin.com/connecteddevicesjci.onmicrosoft.com/B2C_1A_SignIn/api/CombinedSigninAndSignup/confirmed?csrf_token={csrf}&tx=StateProperties={StateProperties}&p=B2C_1A_SignIn"
        headers = {"Referer": url3, "X-Csrf-Token": csrf}
        # The cookie values are being wrapped in quotes by aiohttp's cookie handling
        # We can pass the raw cookie string in the headers instead
        cookie_str = "; ".join(f"{k}={v}" for k, v in cookie_dict.items())
        headers["Cookie"] = cookie_str
        try:
            async with session.get(url3, headers=headers) as response:
                pass
        except aiohttp.client_exceptions.NonHttpUrlRedirectClientError as e:
            redirect_url = str(e).rsplit("Location: ", maxsplit=1)[-1].split()[0]

    code = ""
    if "code=" in redirect_url:
        code = redirect_url.split("code=")[1]
        if "&" in code:
            code = code.split("&")[0]

    async with aiohttp.ClientSession() as session:
        url4 = "https://connecteddevicesjci.b2clogin.com/te/connecteddevicesjci.onmicrosoft.com/B2C_1A_SignIn/oauth2/v2.0/token"
        data = {
            "client_id": client_id,
            "scope": scope,
            "code": code,
            "redirect_uri": redirect_uri,
            "grant_type": "authorization_code",
            "code_verifier": cv,
        }
        async with session.post(url4, data=data) as response:
            res = await response.text()
            return json.loads(res)
